@@ -16,9 +16,12 @@ except ImportError:
     from urllib import unquote as unquote_to_bytes
 from collections import (
     OrderedDict,
-    Sequence,
     defaultdict,
 )
+try:
+    from collections.abc import Sequence
+except ImportError:
+    from collections import Sequence
 try:
     from urllib2 import URLError
 except ImportError:
@@ -28,6 +31,7 @@ try:
 except ImportError:
     from urllib.parse import urlparse
 from .exceptions import (
+    Abort,
     AmbiguousGraftAbort,
     NothingToGraftException,
     OldUpgradeAbort,
@@ -38,7 +42,6 @@ from .util import (
     Seekable,
     byte_diff,
     check_enabled,
-    interval_expired,
     one,
     VersionedDict,
 )
@@ -1299,7 +1302,35 @@ class GitHgStore(object):
             if ref not in (b'refs/notes/cinnabar',):
                 Git.delete_ref(ref)
 
-        if self._metadata_sha1 and update_metadata and not refresh and \
-                interval_expired('fsck', 86400 * 7):
-            logging.warn('Have you run `git cinnabar fsck` recently?')
         GitHgHelper.close(rollback=False)
+
+        # Try to detect issue #207 as early as possible.
+        GitHgHelper._helper = False
+        busted = False
+        from .hg.repo import getbundle_params, stored_files
+        for (node, (parent1, parent2)) in progress_iter(
+                "Checking {} imported file root and head revisions",
+                util.iteritems(stored_files)):
+            if not GitHgHelper.check_file(node, parent1, parent2):
+                busted = True
+                logging.error("Error in file %s" % node)
+        if busted:
+            import json
+            extra = ""
+            if getbundle_params:
+                extra = \
+                    "If it failed, please also copy/paste the following:\n"
+                extra += json.dumps(getbundle_params, sort_keys=True, indent=4)
+            raise Abort(
+                "It seems you have hit a known, rare, and difficult to "
+                "reproduce issue.\n"
+                "Your help would be appreciated.\n"
+                "Please try either `git cinnabar rollback` followed by the "
+                "same command that just\n"
+                "failed, or `git cinnabar reclone`.\n"
+                "Please open a new issue "
+                "(https://github.com/glandium/git-cinnabar/issues/new)\n"
+                "mentioning issue #207 and reporting whether the second "
+                "attempt succeeded.\n" + extra + "\n"
+                "Please keep a copy of this repository."
+            )
